/*------------------------------------------------------------------------
Apo to Biblio:
I.Vlahavas, P.Kefalas, N.Basileiadis, F.Kokkoras, I.Sakellariou
Texnhth Nohmosunh (B' Ekdosh), Ekdoseis Gartaganh, Thessaloniki, 2005
http://aibook.csd.auth.gr
------------------------------------------------------------------------*/
/*-------------------------------------------------------------------
Limited-KQML parser
This is a parser for a small subset of KQML.The performatives are:
- ask-if
- ask-one
- ask-all
- tell
- advertise
- sorry
- broadcast
If the KQML expression is syntactiaccly correct then the parser
creates a Prolog term of the following format:
message(performative(P), sender(S), receiver(R), replyto(RT),
	replywith(RW), language(L), content(C))
which then processes it in order to take action (see respondkqml.pl)
--------------------------------------------------------------------*/
parse_from_file(FileName,Term):-
	see(FileName),
	reads(L),
	seen,!,
	renamefile(FileName),
	parse(Term,L),!.

parse(Term,L):-
	kqml_message(Term,L,[]),
	nl,write('The Message is parsed:'),nl,!,
	write(Term),nl.
parse(error,_):-
	write('Message has a syntax error'),nl,!.

/*-------------------------------------------------------------------
a kqml_message can either be:
- one with performative that has contents, i.e.:
	(<perfomative> 
		:sender <word>
		:receiver <word>
		:in-reply-to <word>
		:reply-with <word>
		:language <word>
		:content <expression>)
- one with performative (see sorry) that has no language and contents
--------------------------------------------------------------------*/
kqml_message(Term) -->
	['('],
	performative_with_content(P),
	parameters_with_content(L),
	[')'],					{Term=..[message,P|L]}.
kqml_message(Term) -->
	['('],
	performative_without_content(P),
	parameters_without_content(L),
	[')'],					{Term=..[message,P|L]}.

performative_with_content(performative(P))-->
	discourse_performative(P).

performative_without_content(performative(P))-->
	intervention_performative(P).
/*-------------------------------------------------------------------
These are the performatives supported by this subset of KQML
--------------------------------------------------------------------*/
discourse_performative('ask-if') --> ['ask-if'].
discourse_performative('ask-all') --> ['ask-all'].
discourse_performative('ask-one') --> ['ask-one'].
discourse_performative('tell') --> ['tell'].
discourse_performative('advertise') --> ['advertise'].
discourse_performative('broadcast') --> ['broadcast'].

intervention_performative('sorry') --> ['sorry'].

/*-------------------------------------------------------------------
The parameters allowed in this subset of KQML
--------------------------------------------------------------------*/
parameters_with_content([S,R,RT,RW,L,C]) --> 
	[':sender'], word(WS),			{S=sender(WS)},
	[':receiver'], word(WR),		{R=receiver(WR)},
	[':in-reply-to'], word(WRT),		{RT=replyto(WRT)},
	[':reply-with'], word(WRW),		{RW=replywith(WRW)},
	[':language'], word(WL),		{L=language(WL)},
	[':content'], content(WC),		{C=content(WC)}.

parameters_without_content([S,R,RT,RW,language(empty),content(empty)]) --> 
	[':sender'], word(WS),			{S=sender(WS)},
	[':receiver'], word(WR),		{R=receiver(WR)},
	[':in-reply-to'], word(WRT),		{RT=replyto(WRT)},
	[':reply-with'], word(WRW),		{RW=replywith(WRW)}.

/*-------------------------------------------------------------------
The content field can be any sting or another KQML message
--------------------------------------------------------------------*/
content(S) --> string(S).
content(S) --> kqml_message(S).

word(W) --> [W].

string(S) --> [S].

/*-------------------------------------------------------------------
Renames the file from name.txt to name.bak after it is read
(there must be another more neat way of doing this!!)
--------------------------------------------------------------------*/

renamefile(F):-
	string_to_atom(SF,F),
	string_to_atom(TXT,'.txt'),
	string_concat(P,TXT,SF),
	string_to_atom(BAK,'.bak'),
	string_concat(P,BAK,NSF),
	string_to_atom(NSF,NF),!,
	rename_file(F,NF),		% Built-in!
	!.
