/*------------------------------------------------------------------------
Apo to Biblio:
I.Vlahavas, P.Kefalas, N.Basileiadis, F.Kokkoras, I.Sakellariou
Texnhth Nohmosunh (B' Ekdosh), Ekdoseis Gartaganh, Thessaloniki, 2005
http://aibook.csd.auth.gr
------------------------------------------------------------------------*/
/*--------------------------------------------------------------------
This is an agent which has a KQML interface
- it reads a message from a file 
	(to read from the network we would require a router)
- parses it to see if it is syntactiaclly correct.
- if it is then produces and outputs the reply message in a file
  in the deirectory of the receiver under a filename corresponding
  to the id of the message.
--------------------------------------------------------------------*/

incoming(FileName):-
	parse_from_file(FileName,Message),!,
	respond_to_file(Message).


/*--------------------------------------------------------------------
Agent STEFFEN is capable of doing DFS and ID.
--------------------------------------------------------------------*/

capable(depth_first_search).
capable(iterative_deepening).

/*--------------------------------------------------------------------
These are specifics to an agent:
get_message_id/1: a possible way to produce message ids
get_agent_id/1: the id of the agent (its name)
speak/1: the languages that the agent can understand
--------------------------------------------------------------------*/

message_id(steffen-1).

get_message_id(NewID):-
	retract(message_id(_-N)),
	N1 is N+1,
	get_agent_id(A),
	assert(message_id(A-N1)),!,
	string_to_atom(S1,A),
	string_to_atom(S2,N1),
	string_concat(S1,S2,S),
	string_to_atom(S,NewID).

get_agent_id(steffen).

speak(prolog).
speak(kqml).

known_agents([petros,john,maria]).

/*-------------------------------------------------------------------
These files are required by the agent:
- parsekqml.pl: the parser of the KQML messages
- readsentence.pl: reads the KQML message character by character
- respondkqml.pl : takes the appropriate action according to message 
--------------------------------------------------------------------*/

:-[parserkqml].
:-[readsentence].
:-[respondkqml].

:- nl,nl,get_agent_id(A),write('**** THIS IS AGENT : '),write(A),nl,nl.